"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var factory_1 = require("../factory");
var ControllerInvocation = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerInvocation, _super);
    function ControllerInvocation(controllerInfo) {
        var _this = _super.call(this) || this;
        _this.controllerInfo = controllerInfo;
        return _this;
    }
    ControllerInvocation.prototype.proceed = function () {
        var _this = this;
        var controller = factory_1.ControllerFactory.resolve(this.controllerInfo, this.facade.dependencyResolver);
        if (this.context.contextType == "HttpRequest")
            controller.request = this.context;
        else
            controller.handshake = this.context;
        controller.invocation = this;
        var method = controller[this.controllerInfo.methodMetaData.name];
        var result;
        if (this.facade.autoValidation && this.controllerInfo.methodMetaData.parameters.length > 0 && !controller.validator.isValid())
            return Promise.resolve(new Core.ActionResult(controller.validator.getValidationErrors(), 400, "application/json"));
        else {
            return new Promise(function (resolve, reject) {
                try {
                    result = method.apply(controller, _this.parameters);
                    resolve(_this.createResult(result));
                }
                catch (e) {
                    reject(e);
                }
            });
        }
    };
    ControllerInvocation.prototype.createResult = function (result) {
        var _this = this;
        return Promise.resolve(result)
            .then(function (awaitedResult) {
            if (awaitedResult instanceof Core.ActionResult)
                return awaitedResult;
            if (_this.controllerInfo.classMetaData.baseClass == "ApiController") {
                return new Core.ActionResult(awaitedResult, undefined, "application/json");
            }
            return new Core.ActionResult(awaitedResult);
        });
    };
    return ControllerInvocation;
}(Core.Invocation));
exports.ControllerInvocation = ControllerInvocation;
